@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # WURFL 2 Python - Wireless Universal Resource File to Python conversion utility
-# Copyright (C) 2004-2009 Armand Lynch
+# Copyright (C) 2004-2010 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -26,14 +26,15 @@ usage: wurfl2python.py [options] WURFL_XML_FILE
 """
 
 import sys
+import codecs
 from time import ctime
 
 from pywurfl.wurflprocessor import DeviceHandler, WurflProcessor, op, main
 
 
-__version__ = "4.4.0a"
+__version__ = "5.0.0a"
 __license__ = "LGPL"
-__copyright__ = "Copyright 2004-2009, Armand Lynch"
+__copyright__ = "Copyright 2004-2010, Armand Lynch"
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
 __url__ = "http://celljam.net/"
@@ -42,44 +43,44 @@ __all__ = ['DeviceSerializer']
 
 class DeviceSerializer(DeviceHandler):
 
-    idstr = "devices.devids[r'''%s'''] = "
-    new_caps = idstr + "devclass(%s, r'''%s''', r'''%s''', %s, {%s})\n"
-    no_caps = idstr + "devclass(%s, r'''%s''', r'''%s''', %s, None)\n"
+    idstr = u"devices.devids[ur'''%s'''] = "
+    new_caps = idstr + u"devclass(%s, ur'''%s''', ur'''%s''', %s, {%s})\n"
+    no_caps = idstr + u"devclass(%s, ur'''%s''', ur'''%s''', %s, None)\n"
     capability_types = {}
 
-    def __str__(self):
+    def encoded_str(self):
         s = []
         c = []
 
-        if self.parent == "root":
-            parent = "None"
+        if self.parent == u"root":
+            parent = u"None"
         else:
-            parent = "devices.devids[r'''%s''']" % self.parent
+            parent = u"devices.devids[ur'''%s''']" % self.parent
 
         for cap in sorted(self.capabilities):
             value = self.capabilities[cap]
             cap_type = self.capability_types.get(cap, None)
             if cap_type == int:
-                c.append("r'''%s''':%d" % (cap, int(value.strip())))
+                c.append("ur'''%s''':%d" % (cap, int(value.strip())))
             elif cap_type == float:
-                c.append("r'''%s''':%f" % (cap, float(value.strip())))
+                c.append("ur'''%s''':%f" % (cap, float(value.strip())))
             elif cap_type == bool:
                 if value.lower() == "true":
-                    c.append("r'''%s''':True" % cap)
+                    c.append("ur'''%s''':True" % cap)
                 elif value.lower() == "false":
-                    c.append("r'''%s''':False" % cap)
+                    c.append("ur'''%s''':False" % cap)
             else:
-                c.append("r'''%s''':r'''%s'''" % (cap, value))
+                c.append("ur'''%s''':ur'''%s'''" % (cap, value))
 
         if c:
             s.append(self.new_caps % (self.devid, parent, self.devid,
                                       self.devua, self.actual_device_root,
-                                      ','.join(c)))
+                                      u','.join(c)))
         else:
             s.append(self.no_caps % (self.devid, parent, self.devid, self.devua,
                                      self.actual_device_root))
 
-        return ''.join(s)
+        return u''.join(s)
 
 
 class WurflPythonWriter(WurflProcessor):
@@ -91,15 +92,15 @@ class WurflPythonWriter(WurflProcessor):
         self.groups = {}
 
     def handle_device(self, devobj):
-        self.outfile.write(str(devobj))
+        self.outfile.write(devobj.encoded_str())
 
     def process_options(self):
         if self.logfile != sys.stderr:
-            self.logfile = file(self.logfile, "wb")
+            self.logfile = codecs.open(self.logfile, "wb", "utf8")
         if self.outfile is not None:
-            self.outfile = file(self.outfile, "wb")
+            self.outfile = codecs.open(self.outfile, "wb", "utf8")
         else:
-            self.outfile = file("wurfl.py", "wb")
+            self.outfile = codecs.open("wurfl.py", "wb", "utf8")
 
     def process_capability(self, devobj, group, capability):
         attrs = capability.attrib
@@ -115,25 +116,26 @@ class WurflPythonWriter(WurflProcessor):
         self.device_handler.capability_types = get_normalized_types(self.tree)
 
         version = self.tree.findtext("*/ver").strip()
-        self.outfile.write('"""\n%s\n\n' % self.outfile.name)
-        self.outfile.write("Generated on: %s\n" % ctime())
-        self.outfile.write("Generated by: wurfl2python.py v%s\n" % __version__)
-        self.outfile.write('"""\n\n')
-        self.outfile.write("__all__ = ['devices', 'wurfl_version']\n")
-        self.outfile.write("wurfl_version = '''%s'''\n\n" % version)
-        self.outfile.write("from pywurfl import *\n\n")
-        self.outfile.write("devices = Devices()\n")
+        self.outfile.write(u"# -*- coding: utf-8 -*-\n")
+        self.outfile.write(u'"""\n%s\n\n' % self.outfile.name)
+        self.outfile.write(u"Generated on: %s\n" % ctime())
+        self.outfile.write(u"Generated by: wurfl2python.py v%s\n" % __version__)
+        self.outfile.write(u'"""\n\n')
+        self.outfile.write(u"__all__ = ['devices', 'wurfl_version']\n")
+        self.outfile.write(u"wurfl_version = '''%s'''\n\n" % version)
+        self.outfile.write(u"from pywurfl import *\n\n")
+        self.outfile.write(u"devices = Devices()\n")
 
     def end_process(self):
         for group, names in self.groups.iteritems():
-            self.outfile.write("devices.devids['''generic'''].groups")
-            self.outfile.write("[r'''%s'''] = [r'''%s''']\n" % (group,
-                                                           "''','''".join(names)))
-        self.outfile.write("\n")
-        self.outfile.write("for device in devices.devids.itervalues():\n")
-        self.outfile.write("    devices.devuas[device.devua] = device\n")
-        self.outfile.write("del device\n")
-        self.outfile.write("\n")
+            self.outfile.write(u"devices.devids['''generic'''].groups")
+            self.outfile.write(u"[ur'''%s'''] = [ur'''%s''']\n" %
+                               (group, u"''', ur'''".join(names)))
+        self.outfile.write(u"\n")
+        self.outfile.write(u"for device in devices.devids.itervalues():\n")
+        self.outfile.write(u"    devices.devuas[device.devua] = device\n")
+        self.outfile.write(u"del device\n")
+        self.outfile.write(u"\n")
 
 
 def get_normalized_types(tree):
