@@ -31,7 +31,7 @@ tsearch = Tokenizer()
 
 def get_info():
 
-    ua = uaentry.get().strip()
+    ua = unicode(uaentry.get().strip())
 
     if type_var.get() == "User Agent":
         try:
@@ -62,7 +62,7 @@ def get_algo():
 def follow_link(event):
     indx = infotext.index("@%d, %d" % (event.x, event.y))
     trange = infotext.tag_prevrange("link", indx)
-    wid = infotext.get(trange[0], trange[1])
+    wid = unicode(infotext.get(trange[0], trange[1]))
 
     try:
         device = devices.select_id(wid)
