@@ -1,5 +1,5 @@
 # pywurfl - Wireless Universal Resource File Tools in Python
-# Copyright (C) 2006-2009 Armand Lynch
+# Copyright (C) 2006-2010 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -32,10 +32,10 @@ from pywurfl.exceptions import (WURFLException, ActualDeviceRootNotFound,
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
-__copyright__ = "Copyright 2006-2009, Armand Lynch"
+__copyright__ = "Copyright 2006-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "6.4.1b"
+__version__ = "7.0.0"
 __all__ = ['devclass', 'Devices']
 
 
@@ -55,9 +55,9 @@ def devclass(parent, devid, devua, actual_device_root, new_caps=None):
     @param parent: A Device class or None.
     @type parent: Device
     @param devid: The device id for the returned class.
-    @type devid: string
+    @type devid: unicode
     @param devua: The user agent for the returned class.
-    @type devua: string
+    @type devua: unicode
     @param actual_device_root: Whether or not the returned class is an actual
                                device.
     @type actual_device_root: boolean
@@ -74,24 +74,28 @@ def devclass(parent, devid, devua, actual_device_root, new_caps=None):
 
             def __str__(self):
                 s = []
-                s.append("User Agent: %s\n" % self.devua)
-                s.append("WURFL ID: %s\n" % self.devid)
-                s.append("Fallbacks: ")
+                s.append(u"User Agent: %s\n" % self.devua)
+                s.append(u"WURFL ID: %s\n" % self.devid)
+                s.append(u"Fallbacks: ")
                 fbs = []
                 base_class = self.__class__.__bases__[0]
                 while base_class is not RootDevice:
                     fbs.append(base_class.devid)
                     base_class = base_class.__bases__[0]
-                s.append("%s\n" % fbs)
-                s.append("Actual Device Root: %s\n\n" % self.actual_device_root)
+                s.append(u"%s\n" % fbs)
+                s.append(u"Actual Device Root: %s\n\n" %
+                         self.actual_device_root)
                 for group in sorted(self.groups.keys()):
-                    s.append("%s\n" % group.upper())
+                    s.append(u"%s\n" % group.upper())
                     for cap in sorted(self.groups[group]):
-                        s.append("%s: %s\n" % (cap, getattr(self, cap)))
-                    s.append("\n")
-                return ''.join(s)
+                        attr = getattr(self, cap)
+                        if isinstance(attr, basestring):
+                            attr = attr.encode('ascii', 'xmlcharrefreplace')
+                        s.append(u"%s: %s\n" % (cap, attr))
+                    s.append(u"\n")
+                return u''.join(s)
 
-        Device.fall_back = 'root'
+        Device.fall_back = u'root'
         Device.groups = {}
     else:
         class Device(parent):
@@ -122,12 +126,12 @@ class Devices(object):
         self.devuas = {}
 
         # Regexes to remove noise from user agents
-        self._noise = (("/SN\d{15}", "/SNXXXXXXXXXXXXXXX"),
-                       ("UP.Link\/.*?(\s|$)", ""),
-                       ("\(via IBM WBI \d+\.\d+\)", ""),
-                       ("GMCC\/\d\.\d", ""))
+        self._noise = ((u"/SN\d{15}", u"/SNXXXXXXXXXXXXXXX"),
+                       (u"UP.Link\/.*?(\s|$)", u""),
+                       (u"\(via IBM WBI \d+\.\d+\)", u""),
+                       (u"GMCC\/\d\.\d", u""))
         self._noise_re = [(re.compile(exp), repl) for exp, repl in self._noise]
-        self._name_test_re = re.compile(r'^(_|[a-z])(_|[a-z]|[0-9])+$')
+        self._name_test_re = re.compile(ur'^(_|[a-z])(_|[a-z]|[0-9])+$')
 
     def find_actual_root(self, device=RootDevice):
         """
@@ -149,7 +153,7 @@ class Devices(object):
         Return a Device object based on the user agent.
 
         @param devua: The device user agent to search for.
-        @type devua: string
+        @type devua: unicode
         @param actual_device_root: Return a device that is an actual device
                                    root
         @type actual_device_root: boolean
@@ -163,6 +167,7 @@ class Devices(object):
         @type instance: boolean
         @raise DeviceNotFound:
         """
+        _unicode_check(u'devua', devua)
         devua = devua.strip()
         device = None
         if devua in self.devuas:
@@ -199,7 +204,7 @@ class Devices(object):
         Return a Device object based on the WURFL ID.
 
         @param devid: The WURFL id to search for.
-        @type devid: string
+        @type devid: unicode
         @param actual_device_root: Return a device that is an actual device
                                    root.
         @param instance: Used to select that you want an instance instead of a
@@ -207,6 +212,7 @@ class Devices(object):
         @type instance: boolean
         @raise DeviceNotFound:
         """
+        _unicode_check(u'devid', devid)
         if devid in self.devids:
             device = self.devids.get(devid)
             if actual_device_root:
@@ -223,28 +229,30 @@ class Devices(object):
         Add a group to the WURFL class hierarchy
         @param group: The group's name. The group name should match this regexp
                       ^(_|[a-z])(_|[a-z]|[0-9])+$
-        @type group: string
+        @type group: unicode
         """
-        self._name_test('group', group)
-        if group not in self.devids['generic'].groups:
-            self.devids['generic'].groups[group] = []
+        _unicode_check(u'group', group)
+        self._name_test(u'group', group)
+        if group not in self.devids[u'generic'].groups:
+            self.devids[u'generic'].groups[group] = []
         else:
-            raise ExistsException("'%s' group exists" % group)
+            raise ExistsException(u"'%s' group exists" % group)
 
     def remove_group(self, group):
         """
         Remove a group and all its capabilities from the WURFL class hierarchy
         @param group: The group name. The group name should match this
                       regex '^[a-z]+(_|[a-z])+$' and be unique.
-        @type group: string
+        @type group: unicode
         """
-        if group not in self.devids['generic'].groups:
-            raise WURFLException("'%s' group not found" % group)
-        caps = self.devids['generic'].groups[group]
-        generic = self.devids['generic']
+        _unicode_check(u'group', group)
+        if group not in self.devids[u'generic'].groups:
+            raise WURFLException(u"'%s' group not found" % group)
+        caps = self.devids[u'generic'].groups[group]
+        generic = self.devids[u'generic']
         for cap in caps:
             self._remove_capability(generic, cap)
-        del self.devids['generic'].groups[group]
+        del self.devids[u'generic'].groups[group]
 
     def _remove_capability(self, device, capability):
         if capability in device.__dict__:
@@ -264,42 +272,46 @@ class Devices(object):
         Add a capability to the WURFL class hierarchy
         @param group: The group name. The group name should match this
                       regex ^(_|[a-z])(_|[a-z]|[0-9])+$
-        @type group: string
+        @type group: unicode
         @param capability: The capability name. The capability name should match
                            this regex ^(_|[a-z])(_|[a-z]|[0-9])+$' and be
                            unique amongst all capabilities.
-        @type capability: string
+        @type capability: unicode
         """
+        _unicode_check(u'group', group)
+        _unicode_check(u'capability', capability)
+        _unicode_check(u'default', default)
         try:
             self.add_group(group)
         except ExistsException:
             # If the group already exists, pass
             pass
 
-        self._name_test('capability', capability)
+        self._name_test(u'capability', capability)
 
-        for grp, caps in self.devids['generic'].groups.iteritems():
+        for grp, caps in self.devids[u'generic'].groups.iteritems():
             if capability in caps:
-                raise ExistsException("'%s' capability exists in group '%s'" %
+                raise ExistsException(u"'%s' capability exists in group '%s'" %
                                       (capability, grp))
         else:
-            self.devids['generic'].groups[group].append(capability)
-            setattr(self.devids['generic'], capability, default)
+            self.devids[u'generic'].groups[group].append(capability)
+            setattr(self.devids[u'generic'], capability, default)
 
     def remove_capability(self, capability):
         """
         Remove a capability from the WURFL class hierarchy
         @param capability: The capability name.
-        @type capability: string
+        @type capability: unicode
         """
-        for group in self.devids['generic'].groups:
-            if capability in self.devids['generic'].groups[group]:
+        _unicode_check(u'capability', capability)
+        for group in self.devids[u'generic'].groups:
+            if capability in self.devids[u'generic'].groups[group]:
                 break
         else:
-            raise WURFLException("'%s' capability not found" % capability)
-        generic = self.devids['generic']
+            raise WURFLException(u"'%s' capability not found" % capability)
+        generic = self.devids[u'generic']
         self._remove_capability(generic, capability)
-        self.devids['generic'].groups[group].remove(capability)
+        self.devids[u'generic'].groups[group].remove(capability)
 
     def add(self, parent, devid, devua, actual_device_root=False,
             capabilities=None):
@@ -307,24 +319,27 @@ class Devices(object):
         Add a device to the WURFL class hierarchy
 
         @param parent: A WURFL ID.
-        @type parent: string
+        @type parent: unicode
         @param devid: The device id for the new device.
-        @type devid: string
+        @type devid: unicode
         @param devua: The user agent for the new device.
-        @type devua: string
+        @type devua: unicode
         @param actual_device_root: Whether or not the new device is an
                                    actual device.
         @type actual_device_root: boolean
         @param capabilities: The new capabilities for the new device class.
         @type capabilities: dict
         """
+        _unicode_check(u'parent', parent)
+        _unicode_check(u'devid', devid)
+        _unicode_check(u'devua', devua)
         if parent not in self.devids:
             raise DeviceNotFound(parent)
         if devid in self.devids:
-            raise ExistsException("'%s' device already exists" % devid)
+            raise ExistsException(u"'%s' device already exists" % devid)
         elif devua in self.devuas:
             dup_devid = self.devuas[devua].devid
-            raise ExistsException("'%s' duplicate user agent with '%s'" %
+            raise ExistsException(u"'%s' duplicate user agent with '%s'" %
                                   (devua, dup_devid))
 
         self.devids[devid] = devclass(self.devids[parent], devid, devua,
@@ -340,26 +355,29 @@ class Devices(object):
 
         @param child: A WURFL ID. The child device cannot be the generic
                       device.
-        @type child: string
+        @type child: unicode
         @param devid: The device id for the new device.
-        @type devid: string
+        @type devid: unicode
         @param devua: The user agent for the new device.
-        @type devua: string
+        @type devua: unicode
         @param actual_device_root: Whether or not the new device is an
                                    actual device.
         @type actual_device_root: boolean
         @param capabilities: The new capabilities for the new device class.
         @type capabilities: dict
         """
-        if child == 'generic':
-            raise WURFLException("cannot insert device before generic device")
+        _unicode_check(u'child', child)
+        _unicode_check(u'devid', devid)
+        _unicode_check(u'devua', devua)
+        if child == u'generic':
+            raise WURFLException(u"cannot insert device before generic device")
         if child not in self.devids:
             raise DeviceNotFound(child)
         if devid in self.devids:
-            raise ExistsException("'%s' device already exists" % devid)
+            raise ExistsException(u"'%s' device already exists" % devid)
         elif devua in self.devuas:
             dup_devid = self.devuas[devua].devid
-            raise ExistsException("'%s' duplicate user agent with '%s'" %
+            raise ExistsException(u"'%s' duplicate user agent with '%s'" %
                                   (devua, dup_devid))
 
         child_device = self.devids[child]
@@ -382,24 +400,27 @@ class Devices(object):
         children attribute is to the inserted device.
 
         @param parent: A WURFL ID.
-        @type parent: string
+        @type parent: unicode
         @param devid: The device id for the new device.
-        @type devid: string
+        @type devid: unicode
         @param devua: The user agent for the new device.
-        @type devua: string
+        @type devua: unicode
         @param actual_device_root: Whether or not the new device is an
                                    actual device.
         @type actual_device_root: boolean
         @param capabilities: The new capabilities for the new device class.
         @type capabilities: dict
         """
+        _unicode_check(u'parent', parent)
+        _unicode_check(u'devid', devid)
+        _unicode_check(u'devua', devua)
         if parent not in self.devids:
             raise DeviceNotFound(parent)
         if devid in self.devids:
-            raise ExistsException("'%s' device already exists" % devid)
+            raise ExistsException(u"'%s' device already exists" % devid)
         elif devua in self.devuas:
             dup_devid = self.devuas[devua].devid
-            raise ExistsException("'%s' duplicate user agent with '%s'" %
+            raise ExistsException(u"'%s' duplicate user agent with '%s'" %
                                   (devua, dup_devid))
 
         parent_device = self.devids[parent]
@@ -420,12 +441,13 @@ class Devices(object):
         Remove a device from the WURFL class hierarchy
 
         @param devid: A WURFL ID. The generic device cannot be removed.
-        @type devid: string
+        @type devid: unicode
         """
+        _unicode_check(u'devid', devid)
         if devid not in self.devids:
             raise DeviceNotFound(devid)
-        if devid == 'generic':
-            raise WURFLException("cannot remove generic device")
+        if devid == u'generic':
+            raise WURFLException(u"cannot remove generic device")
 
         device = self.devids[devid]
         parent_device = device.__bases__[0]
@@ -444,12 +466,13 @@ class Devices(object):
         Remove a device and all of its children from the WURFL class hierarchy
 
         @param devid: A WURFL ID. The generic device cannot be removed.
-        @type devid: string
+        @type devid: unicode
         """
+        _unicode_check(u'devid', devid)
         if devid not in self.devids:
             raise DeviceNotFound(devid)
-        if devid == 'generic':
-            raise WURFLException("cannot remove generic device")
+        if devid == u'generic':
+            raise WURFLException(u"cannot remove generic device")
 
         device = self.devids[devid]
         self._remove_tree(devid)
@@ -461,11 +484,11 @@ class Devices(object):
         """
         Yields all group names
         """
-        return self.devids['generic'].groups.iterkeys()
+        return self.devids[u'generic'].groups.iterkeys()
 
     def _capability_generator(self, return_groups=False):
-        for group in self.devids['generic'].groups:
-            for capability in self.devids['generic'].groups[group]:
+        for group in self.devids[u'generic'].groups:
+            for capability in self.devids[u'generic'].groups[group]:
                 if return_groups:
                     yield (group, capability)
                 else:
@@ -508,7 +531,7 @@ class Devices(object):
         """
         data = []
         [data.append(str(self.devids[x]())) for x in sorted(self.devids)]
-        return md5.new(''.join(data)).hexdigest()
+        return md5.new(u''.join(data)).hexdigest()
 
     def __iter__(self):
         return self.devids.__iter__()
@@ -518,14 +541,14 @@ class Devices(object):
 
     def _normalize_types(self):
         type_set = set()
-        common_caps = ['actual_device_root', 'children', 'devid', 'devua',
-                       'groups', 'fall_back']
+        common_caps = [u'actual_device_root', u'children', u'devid', u'devua',
+                       u'groups', u'fall_back']
 
         for device in self.devids.itervalues():
             for cap in (c for c in device.__dict__ if c not in common_caps and
-                        not c.startswith('_')):
-                if isinstance(getattr(device, cap), str):
-                    type_set.add((cap, str))
+                        not c.startswith(u'_')):
+                if isinstance(getattr(device, cap), unicode):
+                    type_set.add((cap, unicode))
                     try:
                         type_set.remove((cap, float))
                     except KeyError:
@@ -539,7 +562,7 @@ class Devices(object):
                     except KeyError:
                         pass
                 elif isinstance(getattr(device, cap), float):
-                    if (cap, str) not in type_set:
+                    if (cap, unicode) not in type_set:
                         type_set.add((cap, float))
                         try:
                             type_set.remove((cap, int))
@@ -550,7 +573,7 @@ class Devices(object):
                         except KeyError:
                             pass
                 elif isinstance(getattr(device, cap), int):
-                    if ((cap, str) not in type_set and
+                    if ((cap, unicode) not in type_set and
                         (cap, float) not in type_set):
                         if isinstance(getattr(device, cap), bool):
                             if (cap, int) not in type_set:
@@ -573,7 +596,13 @@ class Devices(object):
 
     def _name_test(self, name, value):
         if not self._name_test_re.match(value):
-            msg = "%s '%s' does not conform to regexp "
-            msg += "r'^(_|[a-z])(_|[a-z]|[0-9])+$'"
+            msg = u"%s '%s' does not conform to regexp "
+            msg += u"r'^(_|[a-z])(_|[a-z]|[0-9])+$'"
             raise WURFLException(msg % (name, value))
 
+
+def _unicode_check(name, val):
+    if isinstance(val, basestring):
+        if isinstance(val, str):
+            raise UnicodeError(u"argument '%s' must be a unicode string" % name)
+
